@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 quit ("Lynx not installed") if $wwwget eq "";
 
-$rc = system ("$wwwget -source $url >$tmpfile");
+$rc = system ("$wwwget -source '$url' >$tmpfile");
 
 quit ("HTTP response error.") if $rc;
