@@ -7,9 +7,9 @@
  */
 
 /*
- *  $Date: 1998/12/20 11:17:33 $
+ *  $Date: 2000/01/09 17:31:29 $
  *  $Author: hafner $
- *  $Revision: 1.6 $
+ *  $Revision: 1.7 $
  *  $State: Exp $
  */
 
@@ -22,8 +22,11 @@
 typedef struct key_widget
 {
    GtkWidget	*control;
-   GtkWidget	*meta;
    GtkWidget	*shift;
+   GtkWidget	*mod[5];
+   GtkWidget	*lcontrol;
+   GtkWidget	*lshift;
+   GtkWidget	*lmod[5];
    GtkWidget	*keyname;
    GtkWidget	*combo;
    GtkWidget	*toggle;
@@ -37,6 +40,8 @@ key_dialog (const char *keytext, GtkTooltips *tooltips, const char *infotext,
 	    void (*update) (const key_widget_t *data), void *data);
 void
 update_keywidget (key_widget_t *kw, const char *keytext);
+const char **
+modifier_list (void);
 
 #endif /* not _KEYS_H */
 
