@@ -21,9 +21,9 @@
  */
 
 /*
- *  $Date: 1999/12/03 15:34:24 $
+ *  $Date: 2000/09/03 19:22:03 $
  *  $Author: hafner $
- *  $Revision: 1.32 $
+ *  $Revision: 1.37 $
  *  $State: Exp $
  */
 
@@ -42,16 +42,16 @@
 #include "icons.h"
 #include "menu.h"
 #include "dialog.h"
-#include "themes.h"
+#include "themebrowser.h"
 #include "error.h"
 
-#define HOMEPAGE "http://penguinpowered.com/~ulli/wmakerconf/"
+#define HOMEPAGE "http://ulli.linuxave.net/wmakerconf/"
 
-/*******************************************************************************
+/*****************************************************************************
 
 			    global variables
   
-*******************************************************************************/
+*****************************************************************************/
 
 extern proplist_t *orig_wmaker;		/* original WindowMaker file */
 extern proplist_t *plist_changed;
@@ -61,21 +61,21 @@ extern proplist_t  *orig_rootmenu;	/* original WMRootMenu file */
 extern GtkWidget   *log_text;
 extern GtkTooltips *tooltips;
 
-/*******************************************************************************
+/*****************************************************************************
 
 			    static variables
   
-*******************************************************************************/
+*****************************************************************************/
 
 static GtkWidget *revert_menu = NULL;
 static GtkWidget *save_menu   = NULL;
 static GtkWidget *save_button = NULL;
 
-/*******************************************************************************
+/*****************************************************************************
 
 				prototypes
   
-*******************************************************************************/
+*****************************************************************************/
 
 static void
 toggle_tooltips (GtkWidget *menuitem, gpointer data);
@@ -106,7 +106,7 @@ load (GtkWidget *button, gpointer ptr);
 *******************************************************************************/
 
 GtkWidget *
-make_menubar (GtkWidget *logwindow)
+make_menubar (GtkWidget *logwindow, bool_t show_themes_menu, bool_t noinet)
 /*
  *  Generate menu bar.
  *
@@ -143,11 +143,14 @@ make_menubar (GtkWidget *logwindow)
    gtk_menu_append (GTK_MENU (file_menu), item);
 
 #if defined(HAVE_POPEN) && defined(WWWUPDATE)
-   item = gtk_menu_item_new_with_label (_("Upgrade"));
-   gtk_menu_append (GTK_MENU (file_menu), item);
-   gtk_signal_connect_object (GTK_OBJECT (item), "activate",
-			      GTK_SIGNAL_FUNC (upgrade), NULL);
-
+    if (!noinet)
+    {
+       item = gtk_menu_item_new_with_label (_("Upgrade"));
+       gtk_menu_append (GTK_MENU (file_menu), item);
+       gtk_signal_connect_object (GTK_OBJECT (item), "activate",
+				  GTK_SIGNAL_FUNC (upgrade), NULL);
+    }
+    
    item = gtk_menu_item_new();
    gtk_menu_append (GTK_MENU (file_menu), item);
 #endif /* HAVE_POPEN  && WWWUPDATE */
@@ -179,40 +182,48 @@ make_menubar (GtkWidget *logwindow)
 			      GTK_SIGNAL_FUNC (gtk_widget_show_all),
 			      GTK_OBJECT (logwindow));
 
-   item = gtk_menu_item_new_with_label (_("Update previews"));
-   gtk_menu_append (GTK_MENU (themes_menu), item);
-   gtk_signal_connect (GTK_OBJECT (item), "activate",
-		       GTK_SIGNAL_FUNC (generate_previews), NULL);
+   if (show_themes_menu)
+   {
+      item = gtk_menu_item_new_with_label (_("Update previews"));
+      gtk_menu_append (GTK_MENU (themes_menu), item);
+      gtk_signal_connect (GTK_OBJECT (item), "activate",
+			  GTK_SIGNAL_FUNC (generate_previews), NULL);
+   }
 
 #if defined(NETSCAPE)
 
-   item = gtk_menu_item_new();
-   gtk_menu_append (GTK_MENU (themes_menu), item);
+   if (show_themes_menu && !noinet)
+   {
+      item = gtk_menu_item_new();
+      gtk_menu_append (GTK_MENU (themes_menu), item);
 
-   item = gtk_menu_item_new_with_label (_("wm.themes.org"));
-   gtk_menu_append (GTK_MENU (themes_menu), item);
-   gtk_signal_connect (GTK_OBJECT (item), "activate",
-		       GTK_SIGNAL_FUNC (wwwpage), "http://wm.themes.org");
+      item = gtk_menu_item_new_with_label (_("wm.themes.org"));
+      gtk_menu_append (GTK_MENU (themes_menu), item);
+      gtk_signal_connect (GTK_OBJECT (item), "activate",
+			  GTK_SIGNAL_FUNC (wwwpage), "http://wm.themes.org");
 
-   item = gtk_menu_item_new_with_label (_("Linuxberg themes"));
-   gtk_menu_append (GTK_MENU (themes_menu), item);
-   gtk_signal_connect (GTK_OBJECT (item), "activate",
-		       GTK_SIGNAL_FUNC (wwwpage),
-		       "http://powerlinux.linuxberg.com/themes/");
+      item = gtk_menu_item_new_with_label (_("Linuxberg themes"));
+      gtk_menu_append (GTK_MENU (themes_menu), item);
+      gtk_signal_connect (GTK_OBJECT (item), "activate",
+			  GTK_SIGNAL_FUNC (wwwpage),
+			  "http://www.linuxberg.com/themes/");
 
-   item = gtk_menu_item_new_with_label (_("Largo's themes"));
-   gtk_menu_append (GTK_MENU (themes_menu), item);
-   gtk_signal_connect (GTK_OBJECT (item), "activate",
-		       GTK_SIGNAL_FUNC (wwwpage),
-		       "http://wm.current.nu/themes/index.html");
-   
-   item = gtk_menu_item_new();
-   gtk_menu_append (GTK_MENU (help_menu), item);
+      item = gtk_menu_item_new_with_label (_("Largo's themes"));
+      gtk_menu_append (GTK_MENU (themes_menu), item);
+      gtk_signal_connect (GTK_OBJECT (item), "activate",
+			  GTK_SIGNAL_FUNC (wwwpage),
+			  "http://wm.current.nu/themes/");
+   }
+   if (!noinet)
+   {
+      item = gtk_menu_item_new();
+      gtk_menu_append (GTK_MENU (help_menu), item);
 
-   item = gtk_menu_item_new_with_label (_("Homepage"));
-   gtk_menu_append (GTK_MENU (help_menu), item);
-   gtk_signal_connect (GTK_OBJECT (item), "activate",
-		       GTK_SIGNAL_FUNC (wwwpage), HOMEPAGE);
+      item = gtk_menu_item_new_with_label (_("Homepage"));
+      gtk_menu_append (GTK_MENU (help_menu), item);
+      gtk_signal_connect (GTK_OBJECT (item), "activate",
+			  GTK_SIGNAL_FUNC (wwwpage), HOMEPAGE);
+   }
 #endif /* defined(NETSCAPE) */
 
    menu_bar  = gtk_menu_bar_new ();
@@ -221,9 +232,12 @@ make_menubar (GtkWidget *logwindow)
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), file_menu);
    gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
 
-   item = gtk_menu_item_new_with_label (_("Themes"));
-   gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), themes_menu);
-   gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
+   if (show_themes_menu)
+   {
+      item = gtk_menu_item_new_with_label (_("Themes"));
+      gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), themes_menu);
+      gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
+   }
 
    item = gtk_menu_item_new_with_label (_("Help"));
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), help_menu);
@@ -236,7 +250,7 @@ make_menubar (GtkWidget *logwindow)
 }
 
 GtkWidget *
-make_toolbar (void)
+make_toolbar (bool_t noinet)
 /*
  *  Generate toolbar.
  *
@@ -270,21 +284,23 @@ make_toolbar (void)
 					    (p_array + P_HELP)->mask),
 			    (GtkSignalFunc) readme, NULL);
 #if defined(HAVE_POPEN) && defined(WWWUPDATE)
-   gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-			    NULL,
-			    _("Upgrade wmakerconf-data package via HTTP."),
-			    "Toolbar/Install",
-			    gtk_pixmap_new ((p_array + P_INSTALL)->pixmap,
-					    (p_array + P_INSTALL)->mask),
-			    (GtkSignalFunc) upgrade, NULL);
+    if (!noinet) 
+       gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
+				NULL,
+				_("Upgrade wmakerconf-data package via HTTP."),
+				"Toolbar/Install",
+				gtk_pixmap_new ((p_array + P_INSTALL)->pixmap,
+						(p_array + P_INSTALL)->mask),
+				(GtkSignalFunc) upgrade, NULL);
 #endif /* HAVE_POPEN && WWWUPDATE */
 #if defined(NETSCAPE)
-   gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-			    NULL, _("Wmakerconf homepage"),
-			    "Toolbar/WWW",
-			    gtk_pixmap_new ((p_array + P_HOME)->pixmap,
-					    (p_array + P_HOME)->mask),
-			    (GtkSignalFunc) wwwpage, HOMEPAGE);
+    if (!noinet) 
+       gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
+				NULL, _("Wmakerconf homepage"),
+				"Toolbar/WWW",
+				gtk_pixmap_new ((p_array + P_HOME)->pixmap,
+						(p_array + P_HOME)->mask),
+				(GtkSignalFunc) wwwpage, HOMEPAGE);
 #endif /* defined(NETSCAPE) */
 
    gtk_widget_show_all (toolbar);
@@ -359,15 +375,14 @@ about (GtkWidget *w, gpointer ptr)
       }
       
       gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox),
-			  gtk_label_new ("Copyright (C) 1998, 1999 "
+			  gtk_label_new ("Copyright (C) 1998-2000 "
 					 "Ullrich Hafner"),
 			  FALSE, FALSE, 5);
       gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox),
 			  gtk_label_new ("<hafner@bigfoot.de>"),
 			  FALSE, FALSE, 5);
       gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox),
-			  gtk_label_new ("http://penguinpowered.com/"
-					 "~ulli/wmakerconf"),
+			  gtk_label_new (HOMEPAGE),
 			  FALSE, FALSE, 5);
    }
    if (!GTK_WIDGET_VISIBLE (window))
@@ -550,7 +565,7 @@ toggle_save (bool_t enable, proplist_t *key)
       PLInsertDictionaryEntry (plist_changed, key, pl_yes);
 
       if (strcaseeq (PLGetString (key), "ThemePath"))
-	 themes_list ();
+	 build_theme_list ();
    }
 }
 
