@@ -19,9 +19,9 @@
  */
 
 /*
- *  $Date: 1999/04/08 07:09:57 $
+ *  $Date: 2000/07/19 19:22:50 $
  *  $Author: hafner $
- *  $Revision: 1.9 $
+ *  $Revision: 1.11 $
  *  $State: Exp $
  */
 
@@ -31,9 +31,9 @@
 #include "misc.h"
 
 GtkWidget *
-make_toolbar (void);
+make_toolbar (bool_t noinet);
 GtkWidget *
-make_menubar (GtkWidget *logwindow);
+make_menubar (GtkWidget *logwindow, bool_t themes_menu, bool_t noinet);
 void
 toggle_save (bool_t enable, proplist_t *key);
 void
