@@ -21,9 +21,9 @@
  */
 
 /*
- *  $Date: 1999/12/05 17:27:41 $
+ *  $Date: 2000/09/25 18:15:24 $
  *  $Author: hafner $
- *  $Revision: 1.43 $
+ *  $Revision: 1.49 $
  *  $State: Exp $
  */
 
@@ -190,6 +190,8 @@ static void
 set_entry_text (GtkWidget *entry, gpointer data);
 static void
 import_menu (GtkWidget *entry, gpointer data);
+static void
+set_wmakerconf (GtkWidget *menuitem, gpointer data);
 #ifdef GNOME_TO_WMAKER	    
 static void
 set_gnomemenu (GtkWidget *menuitem, gpointer data);
@@ -241,11 +243,11 @@ save_node (GNode *gnode);
   
 *******************************************************************************/
 
-void
+GtkWidget *
 rootmenu_dialog (GtkWidget *page, GtkTooltips *tooltips)
 {
-   char *filename   = get_gnustep_path ("Defaults/WMRootMenu");
-   proplist_t *menu = read_proplist (filename);
+   char       *filename = get_gnustep_path ("Defaults/WMRootMenu");
+   proplist_t *menu 	= read_proplist (filename);
 
    if (!menu || PLIsString (menu))
    {
@@ -314,14 +316,21 @@ rootmenu_dialog (GtkWidget *page, GtkTooltips *tooltips)
       gtk_widget_show_all (page);
       if (menu)
 	 PLRelease (menu);
-      return;
+      
+      {
+	 GtkWidget *vbox = gtk_vbox_new (FALSE, 5);
+
+	 gtk_box_pack_end (GTK_BOX (page), vbox, FALSE, FALSE, 0);
+	 gtk_widget_show (vbox);
+	 return vbox;
+      }
    }
    else
    {
       gtk_box_pack_start (GTK_BOX (page), real_rootmenu (tooltips, menu),
 			  TRUE, TRUE, 0);
       gtk_widget_show (page);
-      return;
+      return page;
    }
    if (filename)
       Free (filename);
@@ -487,12 +496,10 @@ real_rootmenu (GtkTooltips *tooltips, proplist_t *menu)
    /*
     *  Ctree to show the menu structure
     */
-   ctree = menu_tree = gtk_ctree_new (1, 0);
+   ctree = menu_tree = (GtkWidget *) gtk_dndtree_new (1, 0);
    gtk_object_set_user_data (GTK_OBJECT (ctree), ctree);
    gtk_clist_set_row_height (GTK_CLIST (ctree), 18);
 
-   gtk_clist_set_reorderable (GTK_CLIST (ctree), TRUE);
-   gtk_clist_set_use_drag_icons (GTK_CLIST (ctree), TRUE);
    gtk_signal_connect (GTK_OBJECT (ctree), "tree_move",
 		       GTK_SIGNAL_FUNC (tree_moved), NULL);
 
@@ -1057,18 +1064,7 @@ build_tree (GtkCTreeNode *parent, GtkCTreeNode *sibling, proplist_t *menu)
 	       }
 	       else			/* include single file */
 	       {
-		  FILE *file = fopen (path, "r");
-		  
-		  if (!file)
-		  {
-		     item_destructor (new);
-		     return NULL;	/* ERROR */
-		  }
-		  else
-		  {
-		     fclose (file);
-		     new->action = WM_INCLUDE;
-		  }
+		  new->action = WM_INCLUDE;
 	       }
 	    }
 	 }
@@ -1530,6 +1526,16 @@ context_menu (GtkCTree *ctree, GdkEventButton *event, gpointer ptr)
 	 gtk_signal_connect (GTK_OBJECT(menu_item), "activate",
 			     GTK_SIGNAL_FUNC (insert_menu), NULL);
 
+	 menu_item = gtk_menu_item_new ();
+	 gtk_menu_append (GTK_MENU (menu), menu_item);
+	 gtk_widget_show (menu_item);
+	 
+	 menu_item = gtk_menu_item_new_with_label (_("Insert WMAKERCONF menu"));
+	 gtk_menu_append (GTK_MENU (menu), menu_item);
+	 gtk_widget_show (menu_item);
+	 gtk_signal_connect (GTK_OBJECT(menu_item), "activate",
+			     GTK_SIGNAL_FUNC (set_wmakerconf), NULL);
+
 #ifdef GNOME_TO_WMAKER	    
 	 menu_item = gtk_menu_item_new_with_label (_("Insert GNOME menu"));
 	 gtk_menu_append (GTK_MENU (menu), menu_item);
@@ -1942,6 +1948,104 @@ set_kdemenu (GtkWidget *menuitem, gpointer data)
 #	endif /* KDE_TO_WMAKER */
 #endif /* PERL */
 
+static void
+set_wmakerconf (GtkWidget *menuitem, gpointer data)
+{
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_MENU]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "WM Configuration");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show themes");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Themes");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show appearance");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Appearance");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show menu");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Menu");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_MENU]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Settings");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show misc");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Misc");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show paths");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Paths");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show workspace");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Workspace");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show windows");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Windows");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show effects");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Effects");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show mouse");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Mouse");
+
+   insert_menu (menuitem, "");
+   gtk_object_set_user_data (GTK_OBJECT (menuitem),
+			     (gpointer) cmd_name [WM_EXEC]);
+   set_command (menuitem, NULL);
+   gtk_entry_set_text (GTK_ENTRY (commandline_entry),
+		       "wmakerconf --show shortcuts");
+   gtk_entry_set_text (GTK_ENTRY (itemlabel_entry), "Shortcuts");
+
+   toggle_save (menu_changed = YES, NULL);
+}
+
 #ifdef WMCONFIG
 static void
 set_wmconfig (GtkWidget *menuitem, gpointer data)
@@ -1987,14 +2091,8 @@ import_dynamic_menu (GtkWidget *widget, gpointer data)
    item_t	*item     = gtk_ctree_node_get_row_data (GTK_CTREE (menu_tree),
 							 node);
    char		*tmp_name = get_temporary_file_name ();
-   char		*command;
+   char		*command  = g_strdup_printf ("%s > %s", item->arg, tmp_name);
    
-   if (!tmp_name)
-      tmp_name = g_strdup ("/tmp/dynamic.menu");
-   else
-      tmp_name = g_strdup (tmp_name);
-   
-   command = g_strdup_printf ("%s > %s", item->arg, tmp_name);
    if (system (command))
    {
       dialog_popup (DIALOG_ERROR, NULL, NULL,
@@ -2007,7 +2105,6 @@ import_dynamic_menu (GtkWidget *widget, gpointer data)
    import_menu (widget, tmp_name);
 
    delete_file_or_dir (tmp_name);
-   Free (tmp_name);
    toggle_save (menu_changed = YES, NULL);
 }
 
@@ -2188,25 +2285,20 @@ convert_to_proplist (const char *filename)
    else
    {
       char *command;
+      char *txt = g_strdup_printf (_("Can't convert old style menu\n%s\n"
+				     "to menu in proplist format."), filename);
 
       tmp_name = get_temporary_file_name ();
-      if (!tmp_name)
-	 tmp_name = g_strdup ("/tmp/proplist.menu");
-      else
-	 tmp_name = g_strdup (tmp_name);
+      command  = g_strdup_printf ("%s/wm-oldmenu2new.sh %s %s", PKGDATADIR,
+				  filename, tmp_name);
 
-      command = g_strdup_printf ("%s/wm-oldmenu2new.sh %s %s", PKGDATADIR,
-				 filename, tmp_name);
-
-      if (system (command))
+      if (shell_command (command, txt))
       {
-	 dialog_popup (DIALOG_ERROR, NULL, NULL,
-		       _("Can't convert old style menu\n%s\n"
-			 "to menu in proplist format.\n"
-			 "Please check `stderr' for more details."), filename);
 	 Free (command);
+	 Free (txt);
 	 return NULL;
       }
+      Free (txt);
       Free (command);
       menu = read_proplist (tmp_name);
    }
@@ -2216,8 +2308,6 @@ convert_to_proplist (const char *filename)
       dialog_popup (DIALOG_ERROR, NULL, NULL,
 		    _("Can't parse generated menu file\n`%s'.\n"
 		      "Please check syntax of this file."), tmp_name);
-      if (tmp_name)
-	 Free (tmp_name);
       if (menu)
 	 PLRelease (menu);
       
@@ -2225,10 +2315,7 @@ convert_to_proplist (const char *filename)
    }
    
    if (tmp_name)
-   {
       delete_file_or_dir (tmp_name);
-      Free (tmp_name);
-   }
    
    return menu;
 }
