@@ -21,9 +21,9 @@
  */
 
 /*
- *  $Date: 1999/12/05 17:27:42 $
+ *  $Date: 2000/09/03 19:22:05 $
  *  $Author: hafner $
- *  $Revision: 1.13 $
+ *  $Revision: 1.15 $
  *  $State: Exp $
  */
 
@@ -40,7 +40,7 @@
 #include "texture.h"
 #include "misc.h"
 #include "window.h"
-#include "themes.h"
+#include "themebrowser.h"
 
 #include "error.h"
 
@@ -339,25 +339,21 @@ extended_color_dialog (GtkWidget *page, proplist_t *key, proplist_t *value,
    gtk_container_add (GTK_CONTAINER (frame), hbox);
    gtk_box_pack_start (GTK_BOX (hbox), option_menu, FALSE, FALSE, 0);
 
-   {
-      GtkWidget *button;
-
-      text_color->solid_color = color_button (NULL, key, NULL, NULL, NULL);
-      gtk_box_pack_start (GTK_BOX (hbox), text_color->solid_color,
-			  FALSE, TRUE, 0);
-      text_color->color_set [0] = color_button (NULL, key, NULL,
-						tooltips, _("Foreground"));
-      gtk_box_pack_start (GTK_BOX (hbox), text_color->color_set [0],
-			  FALSE, TRUE, 0);
-      text_color->color_set [1] = color_button (NULL, key, NULL,
-						tooltips, _("Shadow 1"));
-      gtk_box_pack_start (GTK_BOX (hbox), text_color->color_set [1],
-			  FALSE, TRUE, 0);
-      text_color->color_set [2] = color_button (NULL, key, NULL,
-						tooltips, _("Shadow 2"));
-      gtk_box_pack_start (GTK_BOX (hbox), text_color->color_set [2],
-			  FALSE, TRUE, 0);
-   }
+   text_color->solid_color = color_button (NULL, key, NULL, NULL, NULL);
+   gtk_box_pack_start (GTK_BOX (hbox), text_color->solid_color,
+		       FALSE, TRUE, 0);
+   text_color->color_set [0] = color_button (NULL, key, NULL,
+					     tooltips, _("Foreground"));
+   gtk_box_pack_start (GTK_BOX (hbox), text_color->color_set [0],
+		       FALSE, TRUE, 0);
+   text_color->color_set [1] = color_button (NULL, key, NULL,
+					     tooltips, _("Shadow 1"));
+   gtk_box_pack_start (GTK_BOX (hbox), text_color->color_set [1],
+		       FALSE, TRUE, 0);
+   text_color->color_set [2] = color_button (NULL, key, NULL,
+					     tooltips, _("Shadow 2"));
+   gtk_box_pack_start (GTK_BOX (hbox), text_color->color_set [2],
+		       FALSE, TRUE, 0);
    
    gtk_box_pack_start (GTK_BOX (page), frame, FALSE, FALSE, 0);
    gtk_widget_show_all (frame);
