@@ -21,9 +21,9 @@
  */
 
 /*
- *  $Date: 1999/12/05 17:27:43 $
+ *  $Date: 2000/07/19 19:22:51 $
  *  $Author: hafner $
- *  $Revision: 1.70 $
+ *  $Revision: 1.73 $
  *  $State: Exp $
  */
 
@@ -87,6 +87,8 @@ static const char **option_g           = NULL;
   
 *******************************************************************************/
 
+static void
+color_right_button (GtkWidget *dummy, gpointer ptr);
 static void
 update_gradient_preview (wtexture_t *wtexture);
 static void
@@ -1100,8 +1102,8 @@ generate_gradient_dialog (wtexture_t *wtexture)
    wtexture->wgradient [GRADIENT_LIST] = list = gtk_list_new ();
    gtk_list_set_selection_mode (GTK_LIST (list), GTK_SELECTION_BROWSE);
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled), list);
-   gtk_object_set_user_data (GTK_OBJECT (list), wtexture->key);
-         
+   gtk_object_set_user_data (GTK_OBJECT (list), wtexture);
+   
    gtk_signal_connect (GTK_OBJECT (list), "button_press_event",
 		       GTK_SIGNAL_FUNC (list_button_press), wtexture);
 
@@ -1123,7 +1125,7 @@ generate_gradient_dialog (wtexture_t *wtexture)
       
       bbox = gtk_vbutton_box_new ();
       {
-	 proplist_t	*array	= get_texture_array (wtexture);
+	 proplist_t *array = get_texture_array (wtexture);
 
 	 if (!strcaseeq (PLGetString (KEY_OF_TEXTURE), "workspaceback") &&
 	     !strcaseeq (PLGetString (KEY_OF_TEXTURE), "workspacespecificback") &&
@@ -1140,7 +1142,6 @@ generate_gradient_dialog (wtexture_t *wtexture)
 	 if (i == 0)
 	    wtexture->wgradient [GRADIENT_REMOVE] = button;
 	 gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, TRUE, 0);
-	 gtk_object_set_user_data (GTK_OBJECT (button), wtexture);
 	 gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (fct [i]), list);
 	 gtk_tooltips_set_tip (tooltips, button, _(info [i]), NULL);  
@@ -1425,8 +1426,8 @@ show_texture_dialog (wtexture_t *wtexture)
 	 /*
 	  *  Let's normalize the parameters to --from value --to value
 	  */
-	 char 	  *from = "black";
-	 char 	  *to   = "white";
+	 char 	  *from = g_strdup ("black");
+	 char 	  *to   = g_strdup ("white");
 	 unsigned  i 	= 3;
 	 
 	 while (i <= n - 2)
@@ -1435,9 +1436,9 @@ show_texture_dialog (wtexture_t *wtexture)
 	    char *color = PLGetString (PLGetArrayElement (array, i++));
 
 	    if (streq (name, "--from"))
-	       from = color;
+	       from = g_strdup (color);
 	    else if (streq (name, "--to"))
-	       to = color;
+	       to = g_strdup (color);
 	 }
 	 while (PLGetNumberOfElements (array) > 3)
 	    PLRemoveArrayElement (array, 3);
@@ -1445,6 +1446,8 @@ show_texture_dialog (wtexture_t *wtexture)
 	 PLInsertArrayElement (array, PLMakeString (from), 4);
 	 PLInsertArrayElement (array, PLMakeString ("--to"), 5);
 	 PLInsertArrayElement (array, PLMakeString (to), 6);
+	 g_free (from);
+	 g_free (to);
       }
       
       /*
@@ -1481,8 +1484,8 @@ show_texture_dialog (wtexture_t *wtexture)
 	 /*
 	  *  Let's normalize the parameters to --from value --to value
 	  */
-	 char 	  *from = "black";
-	 char 	  *to   = "white";
+	 char 	  *from = g_strdup ("black");
+	 char 	  *to   = g_strdup ("white");
 	 unsigned  i 	= 3;
 	 
 	 while (i <= n - 4)
@@ -1491,9 +1494,9 @@ show_texture_dialog (wtexture_t *wtexture)
 	    char *color = PLGetString (PLGetArrayElement (array, i++));
 
 	    if (streq (name, "--from"))
-	       from = color;
+	       from = g_strdup (color);
 	    else if (streq (name, "--to"))
-	       to = color;
+	       to = g_strdup (color);
 	 }
 	 while (PLGetNumberOfElements (array) > 5)
 	    PLRemoveArrayElement (array, 3);
@@ -1501,6 +1504,8 @@ show_texture_dialog (wtexture_t *wtexture)
 	 PLInsertArrayElement (array, PLMakeString (from), 4);
 	 PLInsertArrayElement (array, PLMakeString ("--to"), 5);
 	 PLInsertArrayElement (array, PLMakeString (to), 6);
+	 g_free (from);
+	 g_free (to);
       }
 
       /*
@@ -2413,7 +2418,7 @@ file_selection (GtkWidget *widget, wtexture_t *wtexture)
 static void
 update_pixmap (GtkWidget *widget, GtkFileSelection *fs)
 {
-   char       *path     = gtk_entry_get_text (GTK_ENTRY (widget));
+   char       *path     = gtk_file_selection_get_filename (fs);
    wtexture_t *wtexture = gtk_object_get_user_data (GTK_OBJECT (fs));
    GtkWidget  *pixmap	= gtk_object_get_data (GTK_OBJECT (fs), "preview");
    char	      *filename;
@@ -2535,12 +2540,57 @@ new_list_item (wtexture_t *wtexture, proplist_t *color)
 
 static char *last_color = NULL;		/* save last color name */
 
+static void
+color_right_button (GtkWidget *dummy, gpointer ptr)
+{
+   color_selection_dialog ((GTK_LIST (ptr)->selection)->data,
+			   gtk_object_get_user_data (GTK_OBJECT (ptr)));
+}
+
 static gint
 list_button_press (GtkWidget *widget, GdkEventButton *event, gpointer ptr)
 {
    if (event->type == GDK_2BUTTON_PRESS)
       color_selection_dialog ((GTK_LIST (widget)->selection)->data,
 			      gtk_object_get_user_data (GTK_OBJECT (widget)));
+   else if (event->button == 3)
+   {
+      static GtkWidget *menu = NULL;
+      GtkWidget	       *menu_item;
+      static GtkWidget *remove_item = NULL;
+
+      if (!menu)
+      {
+	 menu = gtk_menu_new ();
+
+	 menu_item = gtk_menu_item_new_with_label (_("Change"));
+	 gtk_menu_append (GTK_MENU (menu), menu_item);
+	 gtk_widget_show (menu_item);
+	 gtk_signal_connect (GTK_OBJECT(menu_item), "activate",
+			     GTK_SIGNAL_FUNC (color_right_button), widget);
+
+	 remove_item = menu_item = gtk_menu_item_new_with_label (_("Remove"));
+	 gtk_menu_append (GTK_MENU (menu), menu_item);
+	 gtk_widget_show (menu_item);
+	 gtk_signal_connect (GTK_OBJECT(menu_item), "activate",
+			     GTK_SIGNAL_FUNC (list_remove), widget);
+
+	 menu_item = gtk_menu_item_new_with_label (_("Insert"));
+	 gtk_menu_append (GTK_MENU (menu), menu_item);
+	 gtk_widget_show (menu_item);
+	 gtk_signal_connect (GTK_OBJECT(menu_item), "activate",
+			     GTK_SIGNAL_FUNC (list_insert), widget);
+
+	 menu_item = gtk_menu_item_new_with_label (_("Append"));
+	 gtk_menu_append (GTK_MENU (menu), menu_item);
+	 gtk_widget_show (menu_item);
+	 gtk_signal_connect (GTK_OBJECT(menu_item), "activate",
+			     GTK_SIGNAL_FUNC (list_append), widget);
+      }
+      gtk_widget_set_sensitive (remove_item,
+				g_list_length (GTK_LIST (widget)->children) > 2);
+      gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
+   }
    
    return TRUE;
 }
@@ -2558,7 +2608,7 @@ list_insert (GtkWidget *widget, GtkWidget *list)
 {
    GtkWidget  *list_item;
    int	      position;
-   wtexture_t *wtexture = gtk_object_get_user_data (GTK_OBJECT (widget));
+   wtexture_t *wtexture = gtk_object_get_user_data (GTK_OBJECT (list));
    proplist_t *array    = get_texture_array (wtexture);
    proplist_t *element  = PLMakeString (last_color?last_color:g_strdup ("black"));
 
@@ -2601,7 +2651,7 @@ list_append (GtkWidget *widget, GtkWidget *list)
  *	new element is inserted in wmaker array attribute wtexture->key
  */
 {
-   wtexture_t *wtexture = gtk_object_get_user_data (GTK_OBJECT (widget));
+   wtexture_t *wtexture = gtk_object_get_user_data (GTK_OBJECT (list));
    proplist_t *array    = get_texture_array (wtexture);
    proplist_t *element  = PLMakeString (last_color?last_color:g_strdup("black"));
 
@@ -2635,7 +2685,7 @@ list_remove (GtkWidget *widget, GtkWidget *list)
    GList      *tmp_list;
    GList      *clear_list;
    unsigned   i;
-   wtexture_t *wtexture = gtk_object_get_user_data (GTK_OBJECT (widget));
+   wtexture_t *wtexture = gtk_object_get_user_data (GTK_OBJECT (list));
    proplist_t *array    = get_texture_array (wtexture);
    unsigned   position;
 
@@ -2704,8 +2754,11 @@ list_change (GtkWidget *widget, GtkWidget *list)
  *	value of wmaker array attribute wtexture->key is modified.
  */
 {
-   color_selection_dialog ((GTK_LIST (list)->selection)->data,
-			   gtk_object_get_user_data (GTK_OBJECT (list)));
+   wtexture_t *wtexture;
+
+   wtexture = (wtexture_t *) gtk_object_get_user_data (GTK_OBJECT (list));
+   
+   color_selection_dialog ((GTK_LIST (list)->selection)->data, wtexture->key);
 }
 
 static void
