@@ -22,9 +22,9 @@
 
 
 /*
- *  $Date: 1999/12/05 17:27:44 $
+ *  $Date: 2000/09/03 19:22:06 $
  *  $Author: hafner $
- *  $Revision: 1.117 $
+ *  $Revision: 1.123 $
  *  $State: Exp $
  */
 
@@ -79,59 +79,64 @@
 #include "rootmenu.h"
 #include "simple.h"
 #include "path.h"
-#include "themes.h"
+#include "themebrowser.h"
 #include "rimage.h"
 
 #include "error.h"
 
-/*******************************************************************************
+/*****************************************************************************
 
 			     global variables
   
-*******************************************************************************/
+*****************************************************************************/
 
-bool_t		 changed      = NO;
-bool_t		 use_imlib    = NO;
-GtkWidget	 *main_window = NULL;
-GtkTooltips	 *tooltips    = NULL;
-proplist_t	 *windowmaker = NULL;
-proplist_t	 *wmconfig    = NULL;
+bool_t	     changed     = NO;
+bool_t	     use_imlib   = NO;
+GtkWidget   *main_window = NULL;
+GdkColor    *background  = NULL;
+GtkTooltips *tooltips    = NULL;
+proplist_t  *windowmaker = NULL;
+proplist_t  *wmconfig    = NULL;
 
 extern proplist_t *plist_changed;
 extern proplist_t *pl_yes;
 
 GtkWidget *widgetstyle_item = NULL;
 
-/*******************************************************************************
+/*****************************************************************************
 
 			       prototypes
   
-*******************************************************************************/
+*****************************************************************************/
 
 void
 update (gpointer ptr, proplist_t *value);
 static GtkWidget *
-create_page (GtkNotebook *notebook, const char *name, GtkWidget *pixmap,
+create_page (GtkNotebook *notebook, const char *name, const char *pixmap_name,
 	     bool_t horizontal);
 static void
-quit_and_save_popup (char *question, void (*yes_function) (GtkWidget *, gpointer),
+quit_and_save_popup (char *question,
+		     void (*yes_function) (GtkWidget *, gpointer),
 		     gpointer dataptr);
 static void
 switch_subentries (GtkWidget *widget, gpointer ptr);
 static void
 init_option_menu (gpointer data, gpointer user_data);
 static void
-switch_attribute_panel (GtkWidget *widget, gpointer ptr);
+appearance_container (proplist_t *type, proplist_t *thispanel,
+		      GList **options, GtkWidget *box);
 
-/*******************************************************************************
+/*****************************************************************************
 
 			       public code
   
-*******************************************************************************/
+*****************************************************************************/
 
 void
-root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
-	     proplist_t *wmlist)
+root_window (const char *conffile, const node *sections,
+	     int sections_restriction_type, bool_t noinet,
+	     GtkWidget *logwindow,  proplist_t *wmc, 
+	     proplist_t *panel, proplist_t *wmlist)
 /*
  *  Create root window and call the event loop of GTK
  *
@@ -141,47 +146,57 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
    GtkWidget  *box1;
    GtkWidget  *notebook;
 #ifndef FREEBSD_SYSTEM   
-   GtkWidget  *progress_window;
-   GtkWidget  *progress_label;
-   GtkWidget  *progress_bar;
+   GtkWidget  *progress_window = NULL;
+   GtkWidget  *progress_label  = NULL;
+   GtkWidget  *progress_bar    = NULL;
 #endif /* not FREEBSD_SYSTEM */
    GtkWidget  *toolbar;
-   proplist_t *plinfo    = PLMakeString ("Info");
-   proplist_t *pltype    = PLMakeString ("Type");
-   proplist_t *plicon    = PLMakeString ("Icon");
-   proplist_t *pltitle   = PLMakeString ("Title");
-   proplist_t *plwidget  = PLMakeString ("Widget");
-   proplist_t *plcolor   = PLMakeString ("Colorbox");
-   proplist_t *plfont    = PLMakeString ("Fontbox");
-   proplist_t *pltexture = PLMakeString ("Texturebox");
-   proplist_t *plgrp     = PLMakeString ("Group");
-   proplist_t *plrange   = PLMakeString ("Range");
-   proplist_t *plrinfo   = PLMakeString ("RangeInfo");
-   GList      *options	 = NULL;
+   proplist_t *plinfo          = PLMakeString ("Info");
+   proplist_t *pltype          = PLMakeString ("Type");
+   proplist_t *plicon          = PLMakeString ("Icon");
+   proplist_t *pltitle         = PLMakeString ("Title");
+   proplist_t *plwidget        = PLMakeString ("Widget");
+   proplist_t *plcolor         = PLMakeString ("Colorbox");
+   proplist_t *plfont          = PLMakeString ("Fontbox");
+   proplist_t *pltexture       = PLMakeString ("Texturebox");
+   proplist_t *plgrp           = PLMakeString ("Group");
+   proplist_t *plrange         = PLMakeString ("Range");
+   proplist_t *plrinfo         = PLMakeString ("RangeInfo");
+   GList      *options	       = NULL;
+   bool_t      section_found   = NO;
+   GtkWidget  *iconbox 	       = gtk_hbox_new (FALSE, 0);
+   char       *section 	       = NULL;      /* will point to something if only
+					       one section will be shown*/      
+
    
 #if defined(PREVIEWS) && !defined(CONVERT)
    init_wraster_lib ();
 #endif /* defined(PREVIEWS) && !defined(CONVERT) */
-   
+
    windowmaker = wmlist;
    wmconfig    = wmc;
 
-#ifndef FREEBSD_SYSTEM   
+   if (sections_restriction_type == 1 && sections && !sections->next)
+      section = sections->name;
+
    /*
-    *  Progress bar window
+    *  Progress bar window (only if not on FreeBSD and section is not given) 
     */
+#ifndef FREEBSD_SYSTEM   
+   if (!section)
    {
       GtkWidget *vbox;
       
       progress_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
       gtk_window_set_title (GTK_WINDOW (progress_window),
 			    PACKAGE " " VERSION " init ...");
+	 
       gtk_container_set_border_width (GTK_CONTAINER (progress_window), 10);
       gtk_signal_connect (GTK_OBJECT (progress_window), "delete_event",
 			  GTK_SIGNAL_FUNC (gtk_true), NULL);
       gtk_widget_set_usize (progress_window, 250, -1);
       
-      vbox = gtk_vbox_new (FALSE, 5);
+      vbox = gtk_vbox_new (FALSE, 0);
       gtk_container_add (GTK_CONTAINER (progress_window), vbox);
 
       progress_label = gtk_label_new ("");
@@ -201,7 +216,10 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
       gtk_widget_show_all (progress_window);
    }
 #endif /* not FREEBSD_SYSTEM */
-   
+
+   /*
+    *  Fallback if libwraster is not (correctly) installed
+    */
 #ifdef HAVE_IMLIB
    {
       GdkVisual		*gdkvisual;
@@ -221,6 +239,7 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
 	 use_imlib = NO;
    }
 #endif /* HAVE_IMLIB */
+   
    gtk_widget_push_visual (gdk_rgb_get_visual ());
    gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
@@ -228,14 +247,14 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
     *  Main window
     */
    main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-   gtk_window_set_title (GTK_WINDOW (main_window), PACKAGE " " VERSION);
    gtk_signal_connect (GTK_OBJECT (main_window), "delete_event",
 		       GTK_SIGNAL_FUNC (quit), NULL);
    gtk_widget_realize (main_window);
+   background = &main_window->style->bg [GTK_STATE_NORMAL];
 
    init_pixmaps ();
 
-   while (gtk_events_pending())
+   while (gtk_events_pending())		/* FreeBSD workaround */
       gtk_main_iteration();
 
    box1 = gtk_vbox_new (FALSE, 0);
@@ -243,20 +262,30 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
    tooltips = gtk_tooltips_new ();
    gtk_widget_show (box1);
 
-   while (gtk_events_pending())
+   while (gtk_events_pending())		/* FreeBSD workaround */
       gtk_main_iteration();
 
    /*
     *  Menu and toolbox
     */
-   gtk_box_pack_start (GTK_BOX(box1), make_menubar (logwindow), FALSE, TRUE, 0);
-   gtk_box_pack_start (GTK_BOX(box1), toolbar = make_toolbar (), FALSE, TRUE, 0);
+   gtk_box_pack_start (GTK_BOX(box1),
+		       make_menubar (logwindow, section == NULL, noinet),
+		       FALSE, TRUE, 0);
+   if (section)
+   {
+      gtk_box_pack_start (GTK_BOX(box1), iconbox, FALSE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX(iconbox), toolbar = make_toolbar (noinet),
+			  FALSE, TRUE, 0);
+   }
+   else
+      gtk_box_pack_start (GTK_BOX(box1), toolbar = make_toolbar (noinet),
+			  FALSE, TRUE, 0);
    
-   while (gtk_events_pending())
+   while (gtk_events_pending())		/* FreeBSD workaround */
       gtk_main_iteration();
 
    /*
-    *  Groups notebook
+    *  Notebook for all different wmakerconf sections 
     */
    notebook = gtk_notebook_new ();
    gtk_notebook_set_homogeneous_tabs (GTK_NOTEBOOK (notebook), FALSE);
@@ -264,14 +293,15 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
    gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_LEFT);
    gtk_box_pack_start (GTK_BOX (box1), notebook, TRUE, TRUE, 0);
-   gtk_container_set_border_width (GTK_CONTAINER (notebook), 10);
+   gtk_container_set_border_width (GTK_CONTAINER (notebook), section ? 5 : 10);
    gtk_widget_show_all (notebook);
 
-   while (gtk_events_pending())
+   while (gtk_events_pending())		/* FreeBSD workaround */
       gtk_main_iteration();
 
    /*
-    *  Generate notebook page for every group
+    *  Generate notebook page for every section.
+    *  Section information is extracted from config file 'WMWmakerconf'. 
     */
    {
       proplist_t *tab = PLGetAllDictionaryKeys (panel);
@@ -287,55 +317,88 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
 	 GtkWidget  *page;
 
 	 if (!icon || !info || !PLIsString (info) || !PLIsString (icon))
-	    error (_("'%s': parse error."), PKGDATADIR "/WMWmakerconf");
+	    error (_("'%s': parse error."), conffile);
+
 	 /*
-	  *  Create new notbook page
+	  *  Check whether the current section is allowed
 	  */
 	 {
-	    GdkPixmap *pixmap;
-	    GdkBitmap *mask;
-	    char *path = g_strconcat (PKGDATADIR, "/", PLGetString (icon), NULL);
-	    pixmap = gdk_pixmap_create_from_xpm (main_window->window, &mask,
-						 &main_window->style->bg [GTK_STATE_NORMAL], path);
-	    Free (path);
-	    page = create_page (GTK_NOTEBOOK (notebook), D_(PLGetString (info)),
-				gtk_pixmap_new (pixmap, mask), TRUE);
+	    char       *thissecname = PLGetString (thispanel);
+	    const node *cur;
+	    bool_t 	matches;
+	    
+	    for (cur = sections, matches = NO; cur && !matches; cur = cur->next)
+	       if (strcaseeq (thissecname, cur->name))
+		  matches = YES;
+	     
+	    if (sections &&
+		((matches && sections_restriction_type==NO)
+		 || (!matches && sections_restriction_type==YES)))
+		  continue;
+	    /* we are here since this section is allowed */
+	    section_found = TRUE;
 	 }
-	 while (gtk_events_pending())
+
+	 /*
+	  *  Create new notbook page (use icon given in WMWmakerconf)
+	  */
+	 {
+	    char *path = g_strconcat (PKGDATADIR, "/", PLGetString (icon), NULL);
+	    
+	    page = create_page (GTK_NOTEBOOK (notebook), D_(PLGetString (info)),
+				path, TRUE);
+	    Free (path);
+	 }
+	 
+	 while (gtk_events_pending())		/* FreeBSD workaround */
 	    gtk_main_iteration();
+
 	 /*
 	  *  Create container for config widgets
 	  */
 	 {
-	    GtkWidget	*box;
-	    proplist_t	*data;
-	    
-	    if (!PLIsString (type) || strcaseeq (PLGetString (type), "SINGLE"))
-	       box = gtk_vbox_new (FALSE, 5);
-	    else			/* DOUBLE */
-	    {
-	       unsigned *number = Calloc (1, sizeof (unsigned));
-	       
-	       box = gtk_table_new (1, 4, FALSE);
-	       gtk_table_set_row_spacings (GTK_TABLE (box), 5);
-	       gtk_table_set_col_spacings (GTK_TABLE (box), 10);
-	       gtk_container_set_border_width (GTK_CONTAINER (box), 5);
-	       
-	       *number = 1;
-	       gtk_object_set_user_data (GTK_OBJECT (box), number);
-	    }
+	    GtkWidget  *box = gtk_vbox_new (FALSE, 5);
+
 	    gtk_widget_show (box);
 
-	    while (gtk_events_pending())
+	    while (gtk_events_pending())		/* FreeBSD workaround */
 	       gtk_main_iteration();
 
 	    /*
-	     *  Frame or nothing
+	     *  Use different layout if section is given:
+	     *  Notebook border and tabs are invisible, section icon is
+	     *  shown in the top right corner.
 	     */
-	    if (!strcaseeq (PLGetString (thispanel), "Appearance"))
+	    if (section)
 	    {
-	       GtkWidget *frame = gtk_frame_new (NULL);
+	       GdkPixmap *pixmap;
+	       GdkBitmap *mask;
+	       GtkWidget *image;
+	       GtkWidget *hbox 	= gtk_hbox_new (FALSE, 0);
+	       char 	 *path 	= g_strconcat (PKGDATADIR, "/",
+					       PLGetString (icon), NULL);
 	       
+	       pixmap = gdk_pixmap_create_from_xpm (main_window->window, &mask,
+						    background, path);	       
+	       gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
+	       gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
+	       image  = gtk_pixmap_new (pixmap, mask);
+	       gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+	       gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, TRUE, 0);
+	       gtk_box_pack_end (GTK_BOX(iconbox), hbox, FALSE, TRUE, 0);
+	       gtk_widget_show_all (iconbox);
+	    }
+	    /*
+	     *  Config sections (except for Appearance) are
+	     *  surrounded by a frame.
+	     */
+	    if (!strcaseeq (PLGetString (thispanel), "Appearance")
+		&& !strcaseeq (PLGetString (thispanel), "Paths"))
+	    {
+	       GtkWidget *frame;
+
+	       frame = gtk_frame_new (section ? D_(PLGetString (info)) : NULL);
+	       gtk_frame_set_label_align (GTK_FRAME (frame), 0.5, 0.5);
 	       gtk_frame_set_shadow_type (GTK_FRAME (frame),
 					  GTK_SHADOW_ETCHED_IN);
 	       gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
@@ -348,256 +411,50 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
 		       
 	    gtk_widget_show_all (page);
 
-	    while (gtk_events_pending())
+	    while (gtk_events_pending())		/* FreeBSD workaround */
 	       gtk_main_iteration();
 
-	    if (strcaseeq (PLGetString (thispanel), "Menu"))
-	       rootmenu_dialog (box, tooltips);
-	    if (strcaseeq (PLGetString (thispanel), "Themes"))
-	       box = theme_dialog (box);
+	    /*
+	     *  Generate special containers for Menu, Themes and Appearance
+	     */
+	    if (strcaseeq (PLGetString (thispanel), "Menu") &&
+		(!section || strcaseeq (section, "Menu")))
+	       box = rootmenu_dialog (box, tooltips);
+	    if (strcaseeq (PLGetString (thispanel), "Themes") &&
+		(!section || strcaseeq (section, "Themes")))
+	       box = theme_browser (box, section != NULL);
+	    if (strcaseeq (PLGetString (thispanel), "Appearance") &&
+		(!section || strcaseeq (section, "Appearance")))
+	       appearance_container (type, thispanel, &options, box);
 
-	    data = PLMakeData ((unsigned char *) &box, sizeof (GtkWidget *));
-	    PLInsertDictionaryEntry (entry, plwidget, data);
-	    PLRelease (data);
-	    
-	    if (strcaseeq (PLGetString (thispanel), "Appearance"))
+	    /*
+	     *  Store container widget of current panel
+	     */
 	    {
-	       proplist_t *subentries = PLGetAllDictionaryKeys (type);
-	       GtkWidget  *subbook    = gtk_notebook_new ();
-	       unsigned   i;
-	       
-	       gtk_notebook_set_show_tabs (GTK_NOTEBOOK (subbook), TRUE);
-	       gtk_notebook_set_scrollable (GTK_NOTEBOOK (subbook), TRUE);
-	       gtk_notebook_set_tab_pos (GTK_NOTEBOOK (subbook), GTK_POS_TOP);
-	       gtk_notebook_set_homogeneous_tabs (GTK_NOTEBOOK (subbook), TRUE);
-	       gtk_box_pack_start (GTK_BOX (box), subbook, TRUE, TRUE, 0);
-	       gtk_container_set_border_width (GTK_CONTAINER (subbook), 10);
-	       gtk_widget_show_all (subbook);
-	       
-	       while (gtk_events_pending())
-		  gtk_main_iteration();
-	       
-	       for (i = 0; i < PLGetNumberOfElements (subentries); i++)
-	       {
-		  proplist_t *thissub  = PLGetArrayElement (subentries, i);
-		  proplist_t *subentry = PLGetDictionaryEntry (type, thissub);
-		  proplist_t *subinfo  = PLGetDictionaryEntry (subentry, plinfo);
-		  proplist_t *subicon  = PLGetDictionaryEntry (subentry, plicon);
-		  proplist_t *subtype  = PLGetDictionaryEntry (subentry, pltype);
-		  GtkWidget  *subpage;
-		  GtkWidget  *picon = NULL;
-
-		  if (subicon && PLIsString (subicon)) /* subsubsections ? */
-		  {
-		     GdkPixmap *pixmap;
-		     GdkBitmap *mask;
-		     char *path = g_strconcat (PKGDATADIR, "/",
-					       PLGetString (subicon), NULL);
-		     
-		     pixmap = gdk_pixmap_create_from_xpm (main_window->window, &mask, &main_window->style->bg [GTK_STATE_NORMAL], path);
-		     picon  = gtk_pixmap_new (pixmap, mask);
-		     Free (path);
-		  }
-		  subpage = create_page (GTK_NOTEBOOK (subbook),
-					 D_(PLGetString (subinfo)), picon,
-					 FALSE);
-		  data = PLMakeData ((unsigned char *) &subpage,
-				     sizeof (GtkWidget *));
-		  PLInsertDictionaryEntry (subentry, plwidget, data);
-		  PLRelease (data);
-		  
-		  while (gtk_events_pending())
-		     gtk_main_iteration();
-		  
-		  /*
-		   *  Add subboxes for color, texture and font
-		   */
-		  {
-		     GtkWidget  *vbox          = gtk_vbox_new (FALSE, 0);
-		     GtkWidget  *nw_hbox       = gtk_hbox_new (FALSE, 0);
-		     GtkWidget  *color_frame   = gtk_hbox_new (FALSE, 0);
-		     GtkWidget  *texture_frame = gtk_hbox_new (FALSE, 0);
-		     GtkWidget  *color_box     = gtk_hbox_new (FALSE, 0);
-		     GtkWidget  *texture_box   = gtk_vbox_new (FALSE, 0);
-		     GtkWidget  *hbox;
-		     proplist_t *color_data;
-		     proplist_t *texture_data;
-		     proplist_t *font_data;
-		     
-		     gtk_container_add (GTK_CONTAINER (subpage), vbox);
-		     
-		     /*
-		      *  Overall boxes
-		      */
-		     gtk_box_pack_start (GTK_BOX (vbox), nw_hbox,
-					 FALSE, TRUE, 5);
-		     hbox = gtk_hbox_new (FALSE, 0);
-		     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-		     gtk_box_pack_start (GTK_BOX (hbox), color_frame,
-					 TRUE, TRUE, 5);
-		     hbox = gtk_hbox_new (FALSE, 0);
-		     gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
-		     gtk_box_pack_start (GTK_BOX (hbox), texture_frame,
-					 TRUE, TRUE, 5);
-		     while (gtk_events_pending())
-			gtk_main_iteration();
-		     
-		     /*
-		      *  Color box
-		      */
-		     {
-			GtkWidget  *frame = gtk_frame_new (_("Colors"));
-			GtkWidget  *box   = color_box;
-			
-			gtk_box_pack_start (GTK_BOX (color_frame), frame,
-					    TRUE, TRUE, 0);
-			gtk_frame_set_label_align (GTK_FRAME (frame), 0.5, 0.5);
-			gtk_container_add (GTK_CONTAINER (frame), box);
-			gtk_container_set_border_width (GTK_CONTAINER (box), 5);
-			color_data = PLMakeData ((unsigned char *) &box,
-						 sizeof (GtkWidget *));
-			gtk_object_set_user_data (GTK_OBJECT (box),
-						  color_frame);
-		     }
-		     while (gtk_events_pending())
-			gtk_main_iteration();
-		     /*
-		      *  Texture box
-		      */
-		     {
-			GtkWidget  *frame = gtk_frame_new (_("Texture"));
-			GtkWidget  *box   = texture_box;
-
-			gtk_box_pack_start (GTK_BOX (texture_frame), frame,
-					    TRUE, TRUE, 0);
-			gtk_frame_set_label_align (GTK_FRAME (frame), 0.5, 0.5);
-			gtk_container_add (GTK_CONTAINER (frame), box);
-			texture_data = PLMakeData ((unsigned char *) &box,
-						   sizeof (GtkWidget *));
-			gtk_object_set_user_data (GTK_OBJECT (box),
-						  texture_frame);
-		     }
-		     /*
-		      *  Font box
-		      */
-		     {
-			font_data = PLMakeData ((unsigned char *) &nw_hbox,
-						sizeof (GtkWidget *));
-		     }
-		     gtk_widget_show_all (subpage);
-		     gtk_widget_hide (color_frame);
-		     gtk_widget_hide (texture_frame);
-
-		     while (gtk_events_pending())
-			gtk_main_iteration();
-		     
-		     /*
-		      *  Make option box with subsubsections and subboxes
-		      *  for color, texture and font
-		      */
-		     if (subtype && PLIsDictionary (subtype))
-		     {
-			proplist_t *ssentries = PLGetAllDictionaryKeys (subtype);
-			unsigned   k;
-			char	   *path;
-			GtkWidget  *option_menu = gtk_option_menu_new ();
-			GtkWidget  *menu        = gtk_menu_new ();
-
-			path = g_strconcat (PLGetString (thispanel), "/",
-					    PLGetString (thissub), NULL);
-
-			for (k = 0; k < PLGetNumberOfElements (ssentries); k++)
-			{
-			   proplist_t *thisss;
-			   proplist_t *ssentry;
-			   proplist_t *ssinfo;
-			   GtkWidget  *menu_item;
-			   
-			   thisss    = PLGetArrayElement (ssentries, k);
-			   ssentry   = PLGetDictionaryEntry (subtype, thisss);
-			   ssinfo    = PLGetDictionaryEntry (ssentry, plinfo);
-			   menu_item = gtk_menu_item_new_with_label (D_(PLGetString (ssinfo)));
-			   gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-					       GTK_SIGNAL_FUNC (switch_subentries),
-					       path);
-			   gtk_object_set_user_data (GTK_OBJECT (menu_item),
-						     (gpointer) PLGetString (thisss));
-			   gtk_menu_append (GTK_MENU (menu), menu_item);
-			   gtk_widget_show (menu_item);
-
-			   PLInsertDictionaryEntry (ssentry, plcolor,
-						    color_data);
-			   PLInsertDictionaryEntry (ssentry, pltexture,
-						    texture_data);
-			   PLInsertDictionaryEntry (ssentry, plfont,
-						    font_data);
-			   if (strcaseeq (PLGetString (thisss), "Widget"))
-			      widgetstyle_item = menu_item;
-			}
-			gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu),
-						  menu);
-			gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
-			gtk_widget_show (option_menu);
-			options = g_list_append (options, option_menu);
-
-			while (gtk_events_pending())
-			   gtk_main_iteration();
-			
-			{
-			   GtkWidget *frame;
-			   GtkWidget *box = gtk_hbox_new (TRUE, 0);
-
-			   frame = gtk_frame_new (D_(PLGetString (subinfo)));
-			   gtk_frame_set_label_align (GTK_FRAME (frame),
-						      0.5, 0.5);
-			   gtk_container_set_border_width (GTK_CONTAINER (box), 5);
-			   gtk_container_add (GTK_CONTAINER (frame), box);
-			   gtk_box_pack_start (GTK_BOX (box), option_menu,
-					       TRUE, TRUE, 0);
-			   gtk_box_pack_start (GTK_BOX (nw_hbox), frame,
-					       FALSE, TRUE, 5);
-			   gtk_widget_show_all (frame);
-			}
-
-			while (gtk_events_pending())
-			   gtk_main_iteration();
-
-		     }
-		     else
-		     {
-			PLInsertDictionaryEntry (subentry, plcolor,
-						 color_data);
-			PLInsertDictionaryEntry (subentry, pltexture,
-						 texture_data);
-			PLInsertDictionaryEntry (subentry, plfont,
-						 font_data);
-		     }
-		     PLRelease (color_data);
-		     PLRelease (texture_data);
-		     PLRelease (font_data);
-		  }
-	       }
-	       PLRelease (subentries);
+	       proplist_t *data;
+	       data = PLMakeData ((unsigned char *) &box, sizeof (GtkWidget *));
+	       PLInsertDictionaryEntry (entry, plwidget, data);
+	       PLRelease (data);
 	    }
 	 }
       }
       PLRelease (tab);
    }
 
-   while (gtk_events_pending())
+   if (!section_found)
+      error (_("No sections to show!"));
+   
+   while (gtk_events_pending())		/* FreeBSD workaround */
       gtk_main_iteration();
 
    /*
-    *  Add a dialog window for each Window Maker attribute
+    *  Add config widget for each defined Window Maker attribute
     */
    {
       proplist_t *all_keys = PLGetAllDictionaryKeys (wmconfig);
       unsigned   melem     = PLGetNumberOfElements (all_keys);
       unsigned	 n;
 
-      /*
-       *  Generate a dialog widget for every attribute
-       */
       for (n = 0; n < melem; n++)
       {
 	 proplist_t *key    = PLGetArrayElement (all_keys, n);
@@ -617,24 +474,47 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
 	 if (strcaseeq (PLGetString (group), "None"))
 	    continue;
 	 
+#ifndef FREEBSD_SYSTEM   
+	 /*
+	  *  Update of progress bar window
+	  */
+	 if (!section)
+	 {
+	    gtk_label_set_text (GTK_LABEL (progress_label), PLGetString (key));
+	    gtk_progress_bar_update (GTK_PROGRESS_BAR (progress_bar),
+				     n / (double) melem);
+	    gtk_widget_draw (progress_window, NULL);
+	 }
+#endif /* not FREEBSD_SYSTEM */
+	 
+	 while (gtk_events_pending())
+	    gtk_main_iteration();
+
 	 range = PLGetDictionaryEntry (keydef, plrange);
 	 rinfo = PLGetDictionaryEntry (keydef, plrinfo);
 	 info  = PLGetDictionaryEntry (keydef, plinfo);
 	 name  = PLGetString (key);
 	 type  = PLGetString (PLGetDictionaryEntry (keydef, pltype));
 
-#ifndef FREEBSD_SYSTEM   
 	 /*
-	  *  Update of progress bar window
+	  *  We need attributes of actived sections only
 	  */
-	 gtk_label_set_text (GTK_LABEL (progress_label), PLGetString (key));
-	 gtk_progress_bar_update (GTK_PROGRESS_BAR (progress_bar),
-				  n / (double) melem);
-	 gtk_widget_draw (progress_window, NULL);
-#endif /* not FREEBSD_SYSTEM */
-	 
-	 while (gtk_events_pending())
-	    gtk_main_iteration();
+	 {
+	    char        *thissecname = PLGetString (group);
+	    const node*  cur;
+	    bool_t 	 matches;
+	    
+	    for(cur = sections, matches = NO; cur && !matches; cur = cur->next)
+	       if (strncaseeq(thissecname,cur->name,cur->length))
+		  matches = YES;
+	     
+	    if (sections &&
+		((matches && sections_restriction_type==NO)
+		 || (!matches && sections_restriction_type==YES)))
+	       continue;
+	    /* we are here since this section is allowed */
+	    section_found = TRUE;
+	 }
 
 	 /*
 	  *  Get main group and corresponding container
@@ -777,14 +657,16 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
 	 }
 	 else if (strcaseeq ("Pathlist", type))
 	 {
-	    path_dialog (page_box, key, value, tooltips, info);
+	    if (!streq (PLGetString (key), "ThemePath"))
+	       path_dialog (page_box, key, value, tooltips, info);
 	 }
       }
       PLRelease (all_keys);
    }
    
    gtk_widget_set_usize (main_window,
-			 min (gdk_screen_width () - 64, 790),
+			 min (gdk_screen_width () - 64,
+			      790 - (section ? 150 : 0)),
 			 min (gdk_screen_height () - 64, 600));
    
    while (gtk_events_pending())
@@ -792,7 +674,8 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
    
    toggle_save (changed = NO, NULL);
 #ifndef FREEBSD_SYSTEM   
-   gtk_widget_destroy (progress_window);
+   if (!section)
+      gtk_widget_destroy (progress_window);
 #endif /* not FREEBSD_SYSTEM */
 
    PLRelease (plinfo);
@@ -819,7 +702,7 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
    
    while (gtk_events_pending())
       gtk_main_iteration();
-   
+
    gtk_widget_show (main_window);
    
    {
@@ -829,15 +712,14 @@ root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
       if (!(dir = opendir (path)))
       {
 	 dialog_popup (DIALOG_INFO, generate_previews, NULL,
-		       _("Directory $HOME/.wmakerconf does not exist yet.\nThis "
-			 "directory is used to store the previews of all "
+		       _("Directory $HOME/.wmakerconf does not exist yet.\n"
+			 "This directory is used to store the previews of all "
 			 "images.\nShould I generate this directory and the "
 			 "image previews now?"));
       }
       else
 	 closedir (dir);
    }
-
    
    gtk_main ();
    gtk_widget_destroy (main_window);
@@ -853,7 +735,8 @@ check_widgetstyle (void)
    proplist_t *plnewstyle = PLMakeString ("NewStyle");
    proplist_t *value	  = PLGetDictionaryEntry (windowmaker, plnewstyle);
 
-   gtk_widget_set_sensitive (widgetstyle_item, !convert_bool (value));
+   if (widgetstyle_item)
+      gtk_widget_set_sensitive (widgetstyle_item, !convert_bool (value));
    
    PLRelease (plnewstyle);
 }
@@ -996,12 +879,9 @@ make_pixmap (const char *name, int width, int height, GtkWidget *pw)
       }
       else
 #else  /* not HAVE_IMLIB */
-      {
 	 pixmap
 	    = gdk_pixmap_create_from_xpm (main_window->window, &mask,
-					  &main_window->style->bg [GTK_STATE_NORMAL],
-					  PKGDATADIR "/black.xpm"); 
-      }
+					  background, PKGDATADIR "/black.xpm"); 
 #endif /* not HAVE_IMLIB */
       if (!pixmap)			/* pixmap not found or unknown format */
       {
@@ -1186,11 +1066,249 @@ save_config_file (GtkWidget *widget, gpointer ptr)
       gtk_main_quit ();
 }
 
-/*******************************************************************************
+/*****************************************************************************
 
 			       private code
   
-*******************************************************************************/
+*****************************************************************************/
+
+static void
+appearance_container (proplist_t *type, proplist_t *thispanel,
+		      GList **options, GtkWidget *box)
+/*
+ *  Gererate sub-containers for each appearance sub-panel.
+ */
+{
+   proplist_t *subentries = PLGetAllDictionaryKeys (type);
+   GtkWidget  *subbook    = gtk_notebook_new ();
+   proplist_t *plinfo     = PLMakeString ("Info");
+   proplist_t *plfont     = PLMakeString ("Fontbox");
+   proplist_t *pltexture  = PLMakeString ("Texturebox");
+   proplist_t *plcolor    = PLMakeString ("Colorbox");
+   proplist_t *plwidget   = PLMakeString ("Widget");
+   proplist_t *pltype     = PLMakeString ("Type");
+   proplist_t *plicon     = PLMakeString ("Icon");
+   unsigned    i;
+	       
+   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (subbook), TRUE);
+   gtk_notebook_set_scrollable (GTK_NOTEBOOK (subbook), TRUE);
+   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (subbook), GTK_POS_TOP);
+   gtk_notebook_set_homogeneous_tabs (GTK_NOTEBOOK (subbook), TRUE);
+   gtk_box_pack_start (GTK_BOX (box), subbook, TRUE, TRUE, 0);
+   gtk_container_set_border_width (GTK_CONTAINER (subbook), 10);
+   gtk_widget_show_all (subbook);
+	       
+   while (gtk_events_pending())
+      gtk_main_iteration();
+	       
+   for (i = 0; i < PLGetNumberOfElements (subentries); i++)
+   {
+      GtkWidget  *subpage;
+      proplist_t *thissub  = PLGetArrayElement (subentries, i);
+      proplist_t *subentry = PLGetDictionaryEntry (type, thissub);
+      proplist_t *subinfo  = PLGetDictionaryEntry (subentry, plinfo);
+      proplist_t *subicon  = PLGetDictionaryEntry (subentry, plicon);
+      proplist_t *subtype  = PLGetDictionaryEntry (subentry, pltype);
+      char 	 *path     = NULL;
+		  
+      if (subicon && PLIsString (subicon)) /* subsubsections ? */
+	 path = g_strconcat (PKGDATADIR, "/",
+			     PLGetString (subicon), NULL);
+      subpage = create_page (GTK_NOTEBOOK (subbook),
+			     D_(PLGetString (subinfo)), path, FALSE);
+      if (path)
+	 Free (path);
+
+      {
+	 proplist_t *data;
+		     
+	 data = PLMakeData ((unsigned char *) &subpage,
+			    sizeof (GtkWidget *));
+	 PLInsertDictionaryEntry (subentry, plwidget, data);
+	 PLRelease (data);
+      }
+		  
+		  
+      while (gtk_events_pending())
+	 gtk_main_iteration();
+		  
+      /*
+       *  Add subboxes for color, texture and font
+       */
+      {
+	 GtkWidget  *vbox          = gtk_vbox_new (FALSE, 0);
+	 GtkWidget  *nw_hbox       = gtk_hbox_new (FALSE, 0);
+	 GtkWidget  *color_frame   = gtk_hbox_new (FALSE, 0);
+	 GtkWidget  *texture_frame = gtk_hbox_new (FALSE, 0);
+	 GtkWidget  *color_box     = gtk_hbox_new (FALSE, 0);
+	 GtkWidget  *texture_box   = gtk_vbox_new (FALSE, 0);
+	 GtkWidget  *hbox;
+	 proplist_t *color_data;
+	 proplist_t *texture_data;
+	 proplist_t *font_data;
+		     
+	 gtk_container_add (GTK_CONTAINER (subpage), vbox);
+		     
+	 /*
+	  *  Overall boxes
+	  */
+	 gtk_box_pack_start (GTK_BOX (vbox), nw_hbox,
+			     FALSE, TRUE, 5);
+	 hbox = gtk_hbox_new (FALSE, 0);
+	 gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
+	 gtk_box_pack_start (GTK_BOX (hbox), color_frame,
+			     TRUE, TRUE, 5);
+	 hbox = gtk_hbox_new (FALSE, 0);
+	 gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
+	 gtk_box_pack_start (GTK_BOX (hbox), texture_frame,
+			     TRUE, TRUE, 5);
+	 while (gtk_events_pending())
+	    gtk_main_iteration();
+		     
+	 /*
+	  *  Color box
+	  */
+	 {
+	    GtkWidget  *frame = gtk_frame_new (_("Colors"));
+	    GtkWidget  *box   = color_box;
+			
+	    gtk_box_pack_start (GTK_BOX (color_frame), frame,
+				TRUE, TRUE, 0);
+	    gtk_frame_set_label_align (GTK_FRAME (frame), 0.5, 0.5);
+	    gtk_container_add (GTK_CONTAINER (frame), box);
+	    gtk_container_set_border_width (GTK_CONTAINER (box), 5);
+	    color_data = PLMakeData ((unsigned char *) &box,
+				     sizeof (GtkWidget *));
+	    gtk_object_set_user_data (GTK_OBJECT (box),
+				      color_frame);
+	 }
+	 while (gtk_events_pending())
+	    gtk_main_iteration();
+	 /*
+	  *  Texture box
+	  */
+	 {
+	    GtkWidget  *frame = gtk_frame_new (_("Texture"));
+	    GtkWidget  *box   = texture_box;
+
+	    gtk_box_pack_start (GTK_BOX (texture_frame), frame,
+				TRUE, TRUE, 0);
+	    gtk_frame_set_label_align (GTK_FRAME (frame), 0.5, 0.5);
+	    gtk_container_add (GTK_CONTAINER (frame), box);
+	    texture_data = PLMakeData ((unsigned char *) &box,
+				       sizeof (GtkWidget *));
+	    gtk_object_set_user_data (GTK_OBJECT (box),
+				      texture_frame);
+	 }
+	 /*
+	  *  Font box
+	  */
+	 {
+	    font_data = PLMakeData ((unsigned char *) &nw_hbox,
+				    sizeof (GtkWidget *));
+	 }
+	 gtk_widget_show_all (subpage);
+	 gtk_widget_hide (color_frame);
+	 gtk_widget_hide (texture_frame);
+
+	 while (gtk_events_pending())
+	    gtk_main_iteration();
+		     
+	 /*
+	  *  Make option box with subsubsections and subboxes
+	  *  for color, texture and font
+	  */
+	 if (subtype && PLIsDictionary (subtype))
+	 {
+	    proplist_t *ssentries = PLGetAllDictionaryKeys (subtype);
+	    unsigned   k;
+	    char	   *path;
+	    GtkWidget  *option_menu = gtk_option_menu_new ();
+	    GtkWidget  *menu        = gtk_menu_new ();
+
+	    path = g_strconcat (PLGetString (thispanel), "/",
+				PLGetString (thissub), NULL);
+
+	    for (k = 0; k < PLGetNumberOfElements (ssentries); k++)
+	    {
+	       proplist_t *thisss;
+	       proplist_t *ssentry;
+	       proplist_t *ssinfo;
+	       GtkWidget  *menu_item;
+			   
+	       thisss    = PLGetArrayElement (ssentries, k);
+	       ssentry   = PLGetDictionaryEntry (subtype, thisss);
+	       ssinfo    = PLGetDictionaryEntry (ssentry, plinfo);
+	       menu_item = gtk_menu_item_new_with_label (D_(PLGetString (ssinfo)));
+	       gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+				   GTK_SIGNAL_FUNC (switch_subentries),
+				   path);
+	       gtk_object_set_user_data (GTK_OBJECT (menu_item),
+					 (gpointer) PLGetString (thisss));
+	       gtk_menu_append (GTK_MENU (menu), menu_item);
+	       gtk_widget_show (menu_item);
+
+	       PLInsertDictionaryEntry (ssentry, plcolor,
+					color_data);
+	       PLInsertDictionaryEntry (ssentry, pltexture,
+					texture_data);
+	       PLInsertDictionaryEntry (ssentry, plfont,
+					font_data);
+	       if (strcaseeq (PLGetString (thisss), "Widget"))
+		  widgetstyle_item = menu_item;
+	    }
+	    gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu),
+				      menu);
+	    gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
+	    gtk_widget_show (option_menu);
+	    *options = g_list_append (*options, option_menu);
+
+	    while (gtk_events_pending())
+	       gtk_main_iteration();
+			
+	    {
+	       GtkWidget *frame;
+	       GtkWidget *box = gtk_hbox_new (TRUE, 0);
+
+	       frame = gtk_frame_new (D_(PLGetString (subinfo)));
+	       gtk_frame_set_label_align (GTK_FRAME (frame),
+					  0.5, 0.5);
+	       gtk_container_set_border_width (GTK_CONTAINER (box), 5);
+	       gtk_container_add (GTK_CONTAINER (frame), box);
+	       gtk_box_pack_start (GTK_BOX (box), option_menu,
+				   TRUE, TRUE, 0);
+	       gtk_box_pack_start (GTK_BOX (nw_hbox), frame,
+				   FALSE, TRUE, 5);
+	       gtk_widget_show_all (frame);
+	    }
+
+	    while (gtk_events_pending())
+	       gtk_main_iteration();
+
+	 }
+	 else
+	 {
+	    PLInsertDictionaryEntry (subentry, plcolor,
+				     color_data);
+	    PLInsertDictionaryEntry (subentry, pltexture,
+				     texture_data);
+	    PLInsertDictionaryEntry (subentry, plfont,
+				     font_data);
+	 }
+	 PLRelease (color_data);
+	 PLRelease (texture_data);
+	 PLRelease (font_data);
+      }
+   }
+   PLRelease (subentries);
+   PLRelease (plinfo);
+   PLRelease (pltype);
+   PLRelease (plicon);
+   PLRelease (plwidget);
+   PLRelease (plcolor);
+   PLRelease (plfont);
+   PLRelease (pltexture);
+}
 
 static void
 init_option_menu (gpointer data, gpointer user_data)
@@ -1204,7 +1322,7 @@ init_option_menu (gpointer data, gpointer user_data)
 }
 
 static GtkWidget *
-create_page (GtkNotebook *notebook, const char *name, GtkWidget *pixmap,
+create_page (GtkNotebook *notebook, const char *name, const char *pixmap_name,
 	     bool_t horizontal)
 /*
  *  Create a new group notebook page and append it to the given 'notebook'.
@@ -1214,24 +1332,39 @@ create_page (GtkNotebook *notebook, const char *name, GtkWidget *pixmap,
  *	container of label and pixmap
  */
 {
-   GtkWidget *vbox = gtk_vbox_new (FALSE, 5);
+   GtkWidget *vbox 	= gtk_vbox_new (FALSE, 5);
    GtkWidget *label_box = NULL;
+   GtkWidget *scrolled 	= NULL;
 
-   if (name || pixmap)
+   if (name || pixmap_name)
    {
       label_box = horizontal ? gtk_hbox_new (FALSE, 5) : gtk_vbox_new (FALSE, 0);
       gtk_container_set_border_width (GTK_CONTAINER (label_box),
 				      horizontal ? 0 : 5);
-      if (pixmap)
-	 gtk_box_pack_start (GTK_BOX (label_box), pixmap, FALSE, TRUE, 0);
+      if (pixmap_name)
+      {
+	 GdkPixmap *pixmap;
+	 GdkBitmap *mask;
+
+	 pixmap = gdk_pixmap_create_from_xpm (main_window->window, &mask,
+					      background, pixmap_name);
+	 if (pixmap)
+	    gtk_box_pack_start (GTK_BOX (label_box),
+				gtk_pixmap_new (pixmap, mask), FALSE, TRUE, 0);
+      }
       if (name)
 	 gtk_box_pack_start (GTK_BOX (label_box), gtk_label_new (name),
 			     TRUE, TRUE, 0);
       gtk_widget_show_all (label_box);
    }
    
-   gtk_widget_show_all (vbox);
-   gtk_notebook_append_page_menu (notebook, vbox, label_box, NULL);
+   scrolled = gtk_scrolled_window_new (NULL, NULL);
+   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled), 
+				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled),
+					  vbox);    
+   gtk_notebook_append_page_menu (notebook, scrolled, label_box, NULL);
+   gtk_widget_show_all (scrolled);
 
    return vbox;
 }
@@ -1287,14 +1420,3 @@ switch_subentries (GtkWidget *widget, gpointer ptr)
    PLRelease (plgrp);
 }
 
-static void
-switch_attribute_panel (GtkWidget *widget, gpointer ptr)
-/*
- *  Show only widgets of chosen subsubentry
- */
-{
-   GtkNotebook *book = (GtkNotebook *) ptr;
-   GtkWidget   *page = gtk_object_get_user_data (GTK_OBJECT (widget));
-
-   gtk_notebook_set_page (book, gtk_notebook_page_num (book, page));
-}
