@@ -21,9 +21,9 @@
  */
 
 /*
- *  $Date: 1999/04/13 10:15:50 $
+ *  $Date: 2000/09/03 19:22:07 $
  *  $Author: hafner $
- *  $Revision: 1.27 $
+ *  $Revision: 1.30 $
  *  $State: Exp $
  */
 
@@ -33,11 +33,13 @@
 #include <proplist.h>
 #include <gtk/gtk.h>
 #include "misc.h"
-#include "themes.h"
+#include "themebrowser.h"
 
 void
-root_window (GtkWidget *logwindow, proplist_t *wmc, proplist_t *panel,
-	     proplist_t *wmlist);
+root_window (const char *conffile, const node *sections,
+	     int sections_restriction_type, bool_t noinet,
+	     GtkWidget *logwindow,  proplist_t *wmc, 
+	     proplist_t *panel, proplist_t *wmlist);
 char *
 get_pixmap_path (const char *name);
 GtkWidget *
